@@ -128,4 +128,9 @@ def check_port(ip = None, port = None):
     if r == 0:
         print("Port is open")
     else:
-        print("Port is not open")
\ No newline at end of file
+        print("Port is not open")
+
+def check_utc(utc_target) :
+    current_utc = (datetime.now() - datetime.utcnow())
+    current_utc = current_utc.total_seconds()/3600
+    return int(round(utc_target - current_utc,0))
\ No newline at end of file
